<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Fgc\SecurePay\Gateway\Http\Client;

use Exception;
use Fgc\SecurePay\Gateway\Constants;
use Fgc\SecurePay\Helper\Gateway;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Payment\Gateway\Http\ClientInterface;
use Magento\Payment\Gateway\Http\TransferInterface;
use Fgc\SecurePay\Logger\Logger;
use SecurePayApi\Model\Response\CardPayment\PaymentObject;
use SecurePayApi\Model\Response\Error\ErrorObject;
use SecurePayApi\Model\Response\Error\ResponseError;
use SecurePayApi\Request\ApplePay\ApplePayPaymentRequest;
use SecurePayApi\Request\ApplePay\RefundApplePayPaymentRequest;

class ApplePayClientMock implements ClientInterface
{
    protected Gateway $gateway;
    protected RemoteAddress $remoteAddress;
    protected SerializerInterface $serializer;
    private Logger $logger;

    /**
     * @param Logger $logger
     * @param Gateway $gateway
     * @param RemoteAddress $remoteAddress
     * @param SerializerInterface $serializer
     */
    public function __construct(
        Logger $logger,
        Gateway $gateway,
        RemoteAddress $remoteAddress,
        SerializerInterface $serializer
    ) {
        $this->logger = $logger;
        $this->gateway = $gateway;
        $this->remoteAddress = $remoteAddress;
        $this->serializer = $serializer;
    }

    /**
     * Places request to gateway. Returns result as ENV array
     *
     * @param TransferInterface $transferObject
     * @return array
     * @throws LocalizedException
     */
    public function placeRequest(TransferInterface $transferObject): array
    {
        $data = $transferObject->getBody();
        $data = $this->prepareRequestData($data);

        $this->logger->debug('Start ApplePay Request Processing');
        if ($this->gateway->isAvailable()) {
            if ($data[Constants::KEY_ORDER_TYPE] == Constants::ORDER_TYPE_SALE) {
                $request = new ApplePayPaymentRequest(
                    $this->gateway->isLive(),
                    $this->gateway->getCredential(),
                    [
                        'amount' => $data[Constants::KEY_AMOUNT],
                        'currency' => $data[Constants::KEY_CURRENCY],
                        'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
                        'ip' => $this->remoteAddress->getRemoteAddress(),
                        'orderId' => $data[Constants::KEY_ORDER_ID],
                        'paymentData' => $this->serializer->unserialize($data[Constants::KEY_PAYMENT_DATA]),
                        'metadata' => $this->gateway->getMetadata(),
                    ]
                );
            } elseif ($data[Constants::KEY_ORDER_TYPE] == Constants::ORDER_TYPE_REFUND) {
                $this->logger->debug('Start ApplePay Refund');
                $request = new RefundApplePayPaymentRequest(
                    $this->gateway->isLive(),
                    $this->gateway->getCredential(),
                    [
                        'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
                        'amount' => $data[Constants::KEY_AMOUNT],
                        'ip' => $this->remoteAddress->getRemoteAddress(),
                        'metadata' => $this->gateway->getMetadata(),
                    ],
                    $data[Constants::KEY_ORDER_ID]
                );
            } else {
                throw new LocalizedException(
                    __('Unknown requested transaction type: %1', $data[Constants::KEY_ORDER_TYPE])
                );
            }

            $this->logger->debug('Request url: ' . $request->getEndpoint());
            $this->logger->debug('Request data: ' . $this->serializer->serialize($request->getRequestData()));

            try {
                /** @var PaymentObject|ErrorObject $transactionResult */
                $transactionResult = $request->execute();
            } catch (Exception $e) {
                $this->logger->debug($e->getMessage());
                throw new LocalizedException(__('An error occurred in the transaction process. Please try again.'));
            }

            $this->logger->debug(
                'Gateway response: ' . $this->serializer->serialize(($transactionResult->toArray()))
            );

            if ($transactionResult instanceof ResponseError) {
                throw new LocalizedException(
                    __('Error happens with transaction process: %1', $transactionResult->getFirstError()->getDetail())
                );
            }
            if ($transactionResult->getStatus() == 'paid') {
                $data[Constants::KEY_TRANSACTION_RESULT] = $transactionResult->toArray();
            } else {
                throw new LocalizedException(
                    __('Error happens with transaction process: %1', $transactionResult->getGatewayResponseMessage())
                );
            }

            return $data;
        } else {
            throw new LocalizedException(__('An error occurred in the transaction process. Please try again.'));
        }
    }

    protected function prepareRequestData($data)
    {
        if (array_key_exists(Constants::KEY_ORDER_ID, $data)) {
            $data[Constants::KEY_ORDER_ID] = $this->gateway->prepareOrderId($data[Constants::KEY_ORDER_ID]);
        }
        return $data;
    }
}
