<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Fgc\SecurePay\Gateway\Http\Client;

use Exception;
use Fgc\SecurePay\Gateway\Constants;
use Fgc\SecurePay\Helper\Config;
use Fgc\SecurePay\Helper\Gateway;
use Fgc\SecurePay\Helper\QuoteData;
use Fgc\SecurePay\Model\Config\Source\FraudGuardInformation;
use Magento\Checkout\Model\Session;
use Magento\Directory\Api\CountryInformationAcquirerInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Payment\Gateway\Http\ClientInterface;
use Magento\Payment\Gateway\Http\TransferInterface;
use Fgc\SecurePay\Logger\Logger;
use SecurePayApi\Model\Response\CardPayment\PaymentObject;
use SecurePayApi\Model\Response\Error\ErrorObject;
use SecurePayApi\Model\Response\Error\ResponseError;
use SecurePayApi\Request\CardPayment\CancelPreAuthRequest;
use SecurePayApi\Request\CardPayment\CapturePreAuthRequest;
use SecurePayApi\Request\CardPayment\CreatePaymentRequest;
use SecurePayApi\Request\CardPayment\CreatePreAuthRequest;
use SecurePayApi\Request\CardPayment\RefundPaymentRequest;
use SecurePayApi\Request\Request;

class ClientMock implements ClientInterface
{
    protected Gateway $gateway;
    protected Config $configHelper;
    protected SerializerInterface $serializer;
    protected Session $checkoutSession;
    protected CountryInformationAcquirerInterface $countryInformationAcquirer;
    protected QuoteData $quoteData;
    private Logger $logger;
    private RemoteAddress $remoteAddress;

    /**
     * @param Logger $logger
     * @param Gateway $gateway
     * @param Config $configHelper
     * @param RemoteAddress $remoteAddress
     * @param SerializerInterface $serializer
     * @param Session $checkoutSession
     */
    public function __construct(
        Logger $logger,
        Gateway $gateway,
        Config $configHelper,
        RemoteAddress $remoteAddress,
        SerializerInterface $serializer,
        Session $checkoutSession,
        CountryInformationAcquirerInterface $countryInformationAcquirer,
        QuoteData $quoteData
    ) {
        $this->logger = $logger;
        $this->gateway = $gateway;
        $this->configHelper = $configHelper;
        $this->remoteAddress = $remoteAddress;
        $this->serializer = $serializer;
        $this->checkoutSession = $checkoutSession;
        $this->countryInformationAcquirer = $countryInformationAcquirer;
        $this->quoteData = $quoteData;
    }

    /**
     * Places request to gateway. Returns result as ENV array
     *
     * @param TransferInterface $transferObject
     * @return array
     * @throws LocalizedException
     */
    public function placeRequest(TransferInterface $transferObject): array
    {
        $errorMessageWithPlaceHolder = 'An error occurred in the transaction process: %1. Please try again.';

        $data = $transferObject->getBody();
        $data = $this->prepareRequestData($data);

        $this->logger->debug('Start Request Processing');
        if ($this->gateway->isAvailable()) {
            if ($data[Constants::KEY_ORDER_TYPE] == Constants::ORDER_TYPE_SALE) {
                $request = new CreatePaymentRequest(
                    $this->gateway->isLive(),
                    $this->gateway->getCredential(),
                    $this->prepareAdditionalData([
                        'amount' => $data[Constants::KEY_AMOUNT],
                        'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
                        'token' => $data[Constants::KEY_TOKEN],
                        'ip' => $this->remoteAddress->getRemoteAddress(),
                        'orderId' => $data[Constants::KEY_ORDER_ID],
                        'metadata' => $this->gateway->getMetadata(),
                    ], $data)
                );
            } elseif ($data[Constants::KEY_ORDER_TYPE] == Constants::ORDER_TYPE_AUTHORIZE) {
                $request = new CreatePreAuthRequest(
                    $this->gateway->isLive(),
                    $this->gateway->getCredential(),
                    $this->prepareAdditionalData([
                        'amount' => $data[Constants::KEY_AMOUNT],
                        'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
                        'token' => $data[Constants::KEY_TOKEN],
                        'ip' => $this->remoteAddress->getRemoteAddress(),
                        'orderId' => $data[Constants::KEY_ORDER_ID],
                        'metadata' => $this->gateway->getMetadata(),
                    ], $data)
                );
            } elseif ($data[Constants::KEY_ORDER_TYPE] == Constants::ORDER_TYPE_CAPTURE) {
                $request = new CapturePreAuthRequest(
                    $this->gateway->isLive(),
                    $this->gateway->getCredential(),
                    [
                        'amount' => $data[Constants::KEY_AMOUNT],
                        'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
                        'ip' => $this->remoteAddress->getRemoteAddress(),
                        'metadata' => $this->gateway->getMetadata(),
                    ],
                    $data[Constants::KEY_ORDER_ID]
                );
            } elseif ($data[Constants::KEY_ORDER_TYPE] == Constants::ORDER_TYPE_CANCEL) {
                $request = new CancelPreAuthRequest(
                    $this->gateway->isLive(),
                    $this->gateway->getCredential(),
                    [
                        'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
                        'ip' => $this->remoteAddress->getRemoteAddress(),
                        'amount' => $data[Constants::KEY_AMOUNT],
                        'metadata' => $this->gateway->getMetadata(),
                    ],
                    $data[Constants::KEY_ORDER_ID]
                );
            } elseif ($data[Constants::KEY_ORDER_TYPE] == Constants::ORDER_TYPE_REFUND) {
                $request = new RefundPaymentRequest(
                    $this->gateway->isLive(),
                    $this->gateway->getCredential(),
                    [
                        'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
                        'ip' => $this->remoteAddress->getRemoteAddress(),
                        'amount' => $data[Constants::KEY_AMOUNT],
                        'metadata' => $this->gateway->getMetadata(),
                    ],
                    $data[Constants::KEY_ORDER_ID]
                );
            } elseif ($data[Constants::KEY_ORDER_TYPE] == Constants::ORDER_TYPE_VAULT_CAPTURE) {
                $request = new CreatePaymentRequest(
                    $this->gateway->isLive(),
                    $this->gateway->getCredential(),
                    [
                        'amount' => $data[Constants::KEY_AMOUNT],
                        'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
                        'token' => $data[Constants::KEY_TOKEN],
                        'ip' => $this->remoteAddress->getRemoteAddress(),
                        'orderId' => $data[Constants::KEY_ORDER_ID],
                        'customerCode' => $data[Constants::KEY_CUSTOMER_ID],
                        'metadata' => $this->gateway->getMetadata(),
                    ]
                );
            } else {
                throw new LocalizedException(
                    __(
                        $errorMessageWithPlaceHolder,
                        'Unknown requested transaction type: ' . $data[Constants::KEY_ORDER_TYPE]
                    )
                );
            }

            $this->logger->debug('Request url: ' . $request->getEndpoint());
            $this->logRequestData('Request headers', $request->getRequestHeaders());
            $this->logRequestData('Request data', $request->getRequestData());

            /** @var PaymentObject|ErrorObject $transactionResult */
            try {
                $transactionResult = $request->execute();
            } catch (Exception $e) {
                $this->logger->debug('Request error: ' . $e->getMessage());
                throw new LocalizedException(__('An error occurred in the transaction process. Please try again.'));
            }

            $this->logRequestData('Gateway response', $transactionResult->toArray());
            if ($transactionResult instanceof ResponseError) {
                throw new LocalizedException(
                    __($errorMessageWithPlaceHolder, $transactionResult->getFirstError()->getDetail())
                );
            }
            if ($transactionResult->getStatus() == 'paid') {
                $data[Constants::KEY_TRANSACTION_RESULT] = $transactionResult->toArray();
            } else {
                throw new LocalizedException(
                    __($errorMessageWithPlaceHolder, $transactionResult->getGatewayResponseMessage())
                );
            }
            return $data;
        } else {
            $this->logger->debug('Gateway initialization failed.');
            throw new LocalizedException(__('An error occurred in the transaction process. Please try again.'));
        }
    }

    protected function prepareRequestData($data)
    {
        if (array_key_exists(Constants::KEY_ORDER_ID, $data)) {
            $data[Constants::KEY_ORDER_ID] = $this->gateway->prepareOrderId($data[Constants::KEY_ORDER_ID]);
        }
        return $data;
    }

    protected function prepareAdditionalData($data, $baseData)
    {
        if ($this->configHelper->isEnableDcc()) {
            $data['dccDetails']['initiatedOrderId'] = $this->quoteData->getDccOrderId();
            if (isset($baseData[Constants::KEY_CURRENCY])) {
                $data['currency'] = $baseData[Constants::KEY_CURRENCY];
            }
        } elseif ($this->configHelper->isEnableThreeD()) {
            $data['threedSecureDetails']['initiatedOrderId'] = $this->quoteData->getThreeDOrderId();
        } elseif ($this->configHelper->isEnableFraudGuard()) {
            $data['fraudCheckDetails'] = $this->getFraudGuardData();
        }
        return $data;
    }

    protected function getFraudGuardData()
    {
        $result = [
            'fraudCheckType' => 'FRAUD_GUARD',
        ];

        $fraudGuardInformation = $this->configHelper->getFraudGuardInformation();

        if (in_array(FraudGuardInformation::CUSTOMER, $fraudGuardInformation)) {
            $result['customerDetails']['emailAddress'] = $this->checkoutSession->getQuote()->getCustomerEmail();
        }
        if (in_array(FraudGuardInformation::SHIPPING, $fraudGuardInformation)) {
            $shippingAddress = $this->checkoutSession->getQuote()->getShippingAddress();
            $countryId = $shippingAddress->getCountryId();
            $country = $this->countryInformationAcquirer->getCountryInfo($countryId);
            $result['shippingAddress'] = [
                'firstName' => $shippingAddress->getFirstname(),
                'lastName' => $shippingAddress->getLastname(),
                'city' => $shippingAddress->getCity(),
                'postcode' => $shippingAddress->getPostcode(),
                'countryCode' => $country->getThreeLetterAbbreviation(),
            ];
        }
        if (in_array(FraudGuardInformation::BILLING, $fraudGuardInformation)) {
            $countryId = $this->checkoutSession->getQuote()->getBillingAddress()->getCountryId();
            $country = $this->countryInformationAcquirer->getCountryInfo($countryId);
            $result['billingAddress']['countryCode'] = $country->getThreeLetterAbbreviation();
        }

        return $result;
    }

    protected function logRequestData($name, $data)
    {
        $sensitiveKeys = [Request::HEADER_AUTHORIZATION, 'token', 'ip'];
        foreach ($sensitiveKeys as $key) {
            if (is_array($data) && array_key_exists($key, $data)) {
                if (!is_string($data[$key])) {
                    $data[$key] = $this->serializer->serialize($data[$key]);
                }
                $data[$key] = substr($data[$key], 0, 10) . '...';
            }
        }
        $this->logger->debug($name . ': ' . $this->serializer->serialize($data));
    }
}
