<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Fgc\SecurePay\Gateway\Response;

use DateInterval;
use DateTime;
use DateTimeZone;
use Exception;
use Fgc\SecurePay\Gateway\Constants;
use Fgc\SecurePay\Helper\Gateway;
use Fgc\SecurePay\Logger\Logger;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Framework\Message\ManagerInterface;
use Magento\Payment\Gateway\Data\PaymentDataObject;
use Magento\Payment\Gateway\Response\HandlerInterface;
use Magento\Payment\Model\InfoInterface;
use Magento\Sales\Api\Data\OrderPaymentExtensionInterface;
use Magento\Sales\Api\Data\OrderPaymentExtensionInterfaceFactory;
use Magento\Vault\Api\Data\PaymentTokenFactoryInterface;
use Magento\Vault\Api\Data\PaymentTokenInterface;
use SecurePayApi\Model\Response\CardPayment\PaymentInstrumentObject;
use SecurePayApi\Model\Response\Error\ResponseError;
use SecurePayApi\Request\CardPayment\CreatePaymentInstrumentRequest;
use Throwable;

/**
 * Vault Details Handler
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class VaultDetailsHandler implements HandlerInterface
{
    protected Gateway $gateway;
    protected Logger $logger;
    protected ManagerInterface $messageManager;
    private $paymentTokenFactory;
    private $cardMapper = [
        'visa' => 'VI',
        'mastercard' => 'MC',
        'amex' => 'AE',
        'diners' => 'DI',
    ];
    private $paymentExtensionFactory;
    private $remoteAddress;

    public function __construct(
        PaymentTokenFactoryInterface $paymentTokenFactory,
        OrderPaymentExtensionInterfaceFactory $paymentExtensionFactory,
        Gateway $gateway,
        RemoteAddress $remoteAddress,
        ManagerInterface $messageManager,
        Logger $logger
    ) {
        $this->paymentTokenFactory = $paymentTokenFactory;
        $this->paymentExtensionFactory = $paymentExtensionFactory;
        $this->remoteAddress = $remoteAddress;
        $this->gateway = $gateway;
        $this->logger = $logger;
        $this->messageManager = $messageManager;
    }

    /**
     * Handles response
     *
     * @param array $handlingSubject
     * @param array $response
     * @return void
     */
    public function handle(array $handlingSubject, array $response)
    {
        if ($response[Constants::STORE_IN_VAULT_ON_SUCCESS] && isset($response[Constants::KEY_CARD_DETAILS]['is_active_payment_token_enabler'])) {
            $token = $response[Constants::KEY_TOKEN];
            /** @var PaymentDataObject $paymentDO */
            $paymentDO = $handlingSubject['payment'];
            /** @var $payment Payment */
            $payment = $paymentDO->getPayment();

            if ($this->gateway->isAvailable()) {
                try {
                    $this->logger->debug('Start Creating Payment Instrument');

                    $request = new CreatePaymentInstrumentRequest(
                        $this->gateway->isLive(),
                        $this->gateway->getCredential(),
                        $paymentDO->getOrder()->getCustomerId(),
                        $token,
                        $this->remoteAddress->getRemoteAddress(),
                    );
                    /** @var PaymentInstrumentObject $instrument */
                    $instrument = $request->execute();

                    if ($instrument instanceof ResponseError) {
                        throw new Exception(
                            sprintf(
                                "Response Error: %sRequest Data: %s",
                                json_encode($instrument->toArray()),
                                json_encode($request->getRequestData())
                            )
                        );
                    }
                    $paymentToken = $this->getVaultPaymentToken($instrument);

                    $extensionAttributes = $this->getExtensionAttributes($payment);
                    $extensionAttributes->setVaultPaymentToken($paymentToken);
                } catch (Throwable $e) {
                    $this->logger->debug(
                        'Error Creating Payment Instrument: ' . $e->getMessage()
                    );
                    $this->messageManager->addWarningMessage(
                        __('An error occurred while attempting this transaction, please try again or contact support.')
                    );
                }
            }
        }
    }

    /**
     * @param PaymentInstrumentObject $instrument
     * @return PaymentTokenInterface|null
     * @throws Exception
     */
    protected function getVaultPaymentToken(PaymentInstrumentObject $instrument)
    {
        $token = $instrument->getToken();
        if (empty($token)) {
            return null;
        }

        $paymentToken = $this->paymentTokenFactory->create(PaymentTokenFactoryInterface::TOKEN_TYPE_CREDIT_CARD);
        $paymentToken->setGatewayToken($token);
        $paymentToken->setExpiresAt($this->getExpirationDate($instrument));

        $paymentToken->setTokenDetails(
            $this->convertDetailsToJSON([
                'type' => $this->cardMapper[$instrument->getScheme()],
                'maskedCC' => $instrument->getLast4(),
                'expirationDate' => $instrument->getExpiryMonth() . '/' . $instrument->getExpiryYear(),
            ])
        );

        return $paymentToken;
    }

    /**
     * @param PaymentInstrumentObject $instrument
     * @return string
     * @throws Exception
     */
    private function getExpirationDate(PaymentInstrumentObject $instrument): string
    {
        $expDate = new DateTime(
            $instrument->getExpiryYear()
            . '-'
            . $instrument->getExpiryMonth()
            . '-'
            . '01'
            . ' '
            . '00:00:00',
            new DateTimeZone('UTC')
        );
        $expDate->add(new DateInterval('P1M'));
        return $expDate->format('Y-m-d 00:00:00');
    }

    private function convertDetailsToJSON($details)
    {
        $json = json_encode($details);
        return $json ? $json : '{}';
    }

    /**
     * Get payment extension attributes
     * @param InfoInterface $payment
     * @return OrderPaymentExtensionInterface
     */
    private function getExtensionAttributes(InfoInterface $payment)
    {
        /** @var $payment Payment */
        $extensionAttributes = $payment->getExtensionAttributes();
        if (null === $extensionAttributes) {
            $extensionAttributes = $this->paymentExtensionFactory->create();
            $payment->setExtensionAttributes($extensionAttributes);
        }
        return $extensionAttributes;
    }
}
