<?php

namespace Fgc\SecurePay\Helper;

use Fgc\SecurePay\Model\Config\Source\SecurityOption;
use Magento\Config\Model\ResourceModel\Config as ResourceConfig;
use Magento\CurrencySymbol\Model\System\Currencysymbol;
use Magento\Framework\App\Config\ReinitableConfigInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\View\Asset\Repository;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

class Config
{
    public const BASE_CONFIG_PATH = 'payment';
    public const GROUP_SECURE_PAY = 'securepay';
    public const GROUP_SECURE_PAY_VAULT = 'securepay_vault';
    public const GROUP_APPLEPAY = 'securepay_applepay';

    protected ScopeConfigInterface $scopeConfig;
    protected ResourceConfig $resourceConfig;
    protected StoreManagerInterface $storeManager;
    protected ReinitableConfigInterface $reinitableConfig;
    protected Currencysymbol $currencySymbol;
    protected Repository $assetRepository;

    public function __construct(
        ScopeConfigInterface $scopeConfig,
        ResourceConfig $resourceConfig,
        StoreManagerInterface $storeManager,
        ReinitableConfigInterface $reinitableConfig,
        Currencysymbol $currencySymbol,
        Repository $assetRepository
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->resourceConfig = $resourceConfig;
        $this->storeManager = $storeManager;
        $this->reinitableConfig = $reinitableConfig;
        $this->currencySymbol = $currencySymbol;
        $this->assetRepository = $assetRepository;
    }

    public function isSecurePayActive(): bool
    {
        return (bool)$this->getConfig('active', self::GROUP_SECURE_PAY);
    }

    public function getSecurePayTitle(): string
    {
        return $this->getConfig('title', self::GROUP_SECURE_PAY, '');
    }

    public function isSecurePayLive(): bool
    {
        return !$this->getConfig('testmode', self::GROUP_SECURE_PAY);
    }

    public function isDebug(): bool
    {
        return (bool)$this->getConfig('debug', self::GROUP_SECURE_PAY);
    }

    public function isSecurePayVaultActive(): bool
    {
        return (bool)$this->getConfig('active', self::GROUP_SECURE_PAY_VAULT);
    }

    public function getMerchantCode(): string
    {
        return $this->getConfig('merchant_code', self::GROUP_SECURE_PAY, '');
    }

    public function getClientId(): string
    {
        return $this->getConfig('client_id', self::GROUP_SECURE_PAY, '');
    }

    public function getClientSecret(): string
    {
        return $this->getConfig('client_secret', self::GROUP_SECURE_PAY, '');
    }

    public function getSecurePayPaymentAction(): string
    {
        return $this->getConfig('payment_action', self::GROUP_SECURE_PAY, '');
    }

    public function getStoreUid(): string
    {
        return $this->getConfig('store_id', self::GROUP_SECURE_PAY, '');
    }

    public function getSecurePayAllowedCardTypes(): array
    {
        $result = explode(',', $this->getConfig('allowed_card_types', self::GROUP_SECURE_PAY, ''));
        if ($this->isEnableDcc()) {
            $result = array_filter($result, function ($item) {
                return in_array($item, ['visa', 'mastercard']);
            });
        }

        return $result;
    }

    public function isEnableDcc(): bool
    {
        return (bool)$this->getConfig('dcc', self::GROUP_SECURE_PAY);
    }

    public function getMerchantName(): string
    {
        return $this->getConfig('merchant_name', self::GROUP_SECURE_PAY, '');
    }

    public function isEnableThreeD(): bool
    {
        return !$this->isEnableDcc() &&
            ($this->getConfig('security_option', self::GROUP_SECURE_PAY) == SecurityOption::THREE_D);
    }

    public function isEnableFraudGuard(): bool
    {
        return !$this->isEnableDcc() &&
            ($this->getConfig('security_option', self::GROUP_SECURE_PAY) == SecurityOption::FRAUD_GUARD);
    }

    public function getFraudGuardInformation(): array
    {
        return explode(',', $this->getConfig('fraud_guard_information', self::GROUP_SECURE_PAY, ''));
    }

    public function getToken(): string
    {
        return $this->getConfig('token', self::GROUP_SECURE_PAY, '');
    }

    public function getTokenInvalidTime(): string
    {
        return $this->getConfig('token_invalid_time', self::GROUP_SECURE_PAY, '');
    }

    public function setToken($value)
    {
        $this->writeConfig('token', $value, self::GROUP_SECURE_PAY);
    }

    public function setTokenInvalidTime($value)
    {
        $this->writeConfig('token_invalid_time', $value, self::GROUP_SECURE_PAY);
    }

    public function isApplePayActive(): bool
    {
        return (bool)$this->getConfig('active', self::GROUP_APPLEPAY);
    }

    public function getApplePayAllowedCardTypes(): array
    {
        return explode(',', $this->getConfig('allowed_card_types', self::GROUP_APPLEPAY, ''));
    }

    public function toStoreCurrencyCode($globalCurrencyCode)
    {
        return $this->currencySymbol->getCurrencySymbol($globalCurrencyCode) ?: $globalCurrencyCode;
    }

    public function getLogoUrl(): string
    {
        return $this->assetRepository->getUrl('Fgc_SecurePay::image/logo.png');
    }

    protected function getConfig($name, $group, $default = null)
    {
        return $this->scopeConfig->getValue(
            self::BASE_CONFIG_PATH . '/' . $group . '/' . $name,
            ScopeInterface::SCOPE_STORE
        ) ?? $default;
    }

    protected function writeConfig($name, $value, $group)
    {
        $storeId = $this->storeManager->getStore()->getId();

        $this->resourceConfig->saveConfig(
            self::BASE_CONFIG_PATH . '/' . $group . '/' . $name,
            $value,
            ScopeInterface::SCOPE_STORES,
            $storeId
        );

        $this->reinitableConfig->reinit();
    }
}
