<?php

namespace Fgc\SecurePay\Helper;

use Fgc\SecurePay\Logger\Logger;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\Exception\LocalizedException;
use SecurePayApi\Model\Credential;
use SecurePayApi\Model\Response\ClientCredentialsData;
use SecurePayApi\Request\ClientCredentialsRequest;

class Gateway
{
    protected bool $isLive;
    protected ?Credential $credential;
    protected ?string $merchantCode;
    protected ?string $clientId;
    protected ?string $clientSecret;
    protected ?string $storeId;
    protected ?string $token;
    protected Logger $logger;
    protected Config $configHelper;
    protected ProductMetadataInterface $productMetadata;

    public function __construct(
        Config $configHelper,
        ProductMetadataInterface $productMetadata,
        Logger $logger
    ) {
        $this->configHelper = $configHelper;
        $this->logger = $logger;

        $this->initVariables();
        $this->productMetadata = $productMetadata;
    }

    /**
     * @return Credential
     * @throws LocalizedException
     */
    public function getCredential(): Credential
    {
        if (!$this->credential) {
            throw new LocalizedException(__('Failed to initialize Secure Pay.'));
        }

        return $this->credential;
    }

    /**
     * @return bool
     */
    public function isLive(): bool
    {
        return $this->isLive;
    }

    public function isAvailable(): bool
    {
        return $this->token && $this->credential;
    }

    /**
     * @return string|null
     */
    public function getStoreId(): ?string
    {
        return $this->storeId;
    }

    public function prepareOrderId(string $orderId): string
    {
        return $this->getStoreId() . '-' . $orderId;
    }

    private function initVariables()
    {
        $this->isLive = $this->configHelper->isSecurePayLive();
        $this->merchantCode = $this->configHelper->getMerchantCode();
        $this->clientId = $this->configHelper->getClientId();
        $this->clientSecret = $this->configHelper->getClientSecret();
        $this->storeId = substr($this->configHelper->getStoreUid(), 0, 24);

        if ($this->merchantCode && $this->clientId && $this->clientSecret) {
            $this->initCredentials();
        }
    }

    private function initCredentials()
    {
        $tokenTime = $this->configHelper->getTokenInvalidTime();
        $this->token = $this->configHelper->getToken();
        $now = time();
        if (!$this->token || !$tokenTime || $tokenTime < $now) {
            $this->logger->debug('Credential invalid. Reinitializing.');

            $clientCredentialsRequest = new ClientCredentialsRequest(
                $this->isLive, $this->clientId, $this->clientSecret
            );
            /** @var ClientCredentialsData $credentials */
            $credentials = $clientCredentialsRequest->execute();

            $this->configHelper->setTokenInvalidTime($now + $credentials->getExpiresIn());
            $this->configHelper->setToken($credentials->getAccessToken());

            $this->token = $credentials->getAccessToken();
        }

        $this->credential = new Credential(
            $this->isLive,
            $this->merchantCode,
            $this->clientId,
            $this->clientSecret,
            $this->token
        );
    }

    public function getMetadata()
    {
        return [
            'shoppingCartPlatform' => [
                'name' => $this->productMetadata->getName(),
                'version' => $this->productMetadata->getVersion()
            ]
        ];
    }
}
