<?php

namespace Fgc\SecurePay\Model;

use Exception;
use Fgc\SecurePay\Api\ApplepayInitiatorInterface;
use Fgc\SecurePay\Helper\Gateway;
use Fgc\SecurePay\Logger\Logger;
use Fgc\SecurePay\Model\Data\ApplePayData;
use Fgc\SecurePay\Model\Data\ApplepayDataFactory;
use Magento\Checkout\Model\Session;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Framework\UrlInterface;
use SecurePayApi\Model\Response\ApplePay\InitiateSessionResponseObject;
use SecurePayApi\Model\Response\Error\ResponseError;
use SecurePayApi\Request\ApplePay\InitiateSessionRequest;

class ApplepayInitiator implements ApplepayInitiatorInterface
{
    protected ApplepayDataFactory $applepayDataFactory;
    protected Logger $logger;
    protected Session $checkoutSession;
    protected Gateway $gateway;
    protected RemoteAddress $remoteAddress;
    protected SerializerInterface $serializer;

    public function __construct(
        ApplepayDataFactory $applepayDataFactory,
        Logger $logger,
        Session $checkoutSession,
        Gateway $gateway,
        RemoteAddress $remoteAddress,
        SerializerInterface $serializer
    ) {
        $this->applepayDataFactory = $applepayDataFactory;
        $this->logger = $logger;
        $this->checkoutSession = $checkoutSession;
        $this->gateway = $gateway;
        $this->remoteAddress = $remoteAddress;
        $this->serializer = $serializer;
    }

    /**
     * @inheridoc
     */
    public function initiate(): ApplePayData
    {
        $this->logger->debug('SecurePay Apple Pay initialize begins.');


        $store = $this->checkoutSession->getQuote()->getStore();

        $baseUrl = $store->getBaseUrl(UrlInterface::URL_TYPE_WEB, true);
        $parseResult = parse_url($baseUrl);
        $domain = $parseResult['host'];

        $request = new InitiateSessionRequest(
            $this->gateway->isLive(),
            $this->gateway->getCredential(),
            [
                'domain' => $domain,
                'displayName' => $store->getName(),
                'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
                'ip' => $this->remoteAddress->getRemoteAddress(),
                'metadata' => $this->gateway->getMetadata(),
            ],
        );


        $this->logger->debug(
            'SecurePay Apple Pay initialize request: ' . $this->serializer->serialize($request->getRequestData())
        );

        /** @var InitiateSessionResponseObject $response */
        try {
            $response = $request->execute();
        } catch (Exception $e) {
            $this->logger->debug('SecurePay Apple Pay initialize error: ' . $e->getMessage());
            throw new LocalizedException(__('An error occurred while attempting this transaction, please try again or contact support.'));
        }

        if ($response instanceof ResponseError) {
            $this->logger->debug(
                'SecurePay Apple Pay initialize error: ' .
                $this->serializer->serialize($response->getFirstError()->toArray())
            );
            throw new LocalizedException(__('An error occurred while attempting this transaction, please try again or contact support.'));
        }

        /** @var ApplePayData $result */
        $result = $this->applepayDataFactory->create();
        $result->setSessionObject($this->serializer->serialize($response->getSessionObject()));

        return $result;
    }
}
