<?php

namespace Fgc\SecurePay\Model;

use Exception;
use Fgc\SecurePay\Helper\Config;
use Fgc\SecurePay\Helper\Gateway;
use Fgc\SecurePay\Helper\QuoteData;
use Fgc\SecurePay\Logger\Logger;
use Fgc\SecurePay\Model\Data\ThreeDData;
use Fgc\SecurePay\Model\Data\ThreeDDataFactory;
use Fgc\SecurePay\Api\ThreeDInitiatorInterface;
use Magento\Checkout\Model\Session;
use Magento\Framework\DataObject;
use Magento\Framework\Event\ManagerInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Api\Data\CartItemInterface;
use Magento\Quote\Model\Quote;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory as OrderCollectionFactory;
use Magento\Sales\Model\ResourceModel\Order\Item\CollectionFactory as OrderItemCollectionFactory;
use SecurePayApi\Model\Response\CardPayment\PaymentOrderObject;
use SecurePayApi\Model\Response\Error\ResponseError;
use SecurePayApi\Request\CardPayment\InitiatePaymentOrderRequest;

class ThreeDInitiator implements ThreeDInitiatorInterface
{
    protected Gateway $gateway;
    protected RemoteAddress $remoteAddress;
    protected ThreeDDataFactory $threeDDataFactory;
    protected Session $checkoutSession;
    protected Config $configHelper;
    protected QuoteData $quoteData;
    protected ManagerInterface $eventManager;
    protected SerializerInterface $serializer;
    protected OrderCollectionFactory $orderCollectionFactory;
    protected OrderItemCollectionFactory $orderItemCollectionFactory;
    protected Logger $logger;

    public function __construct(
        Gateway $gateway,
        Config $configHelper,
        RemoteAddress $remoteAddress,
        ThreeDDataFactory $threeDDataFactory,
        Session $checkoutSession,
        QuoteData $quoteData,
        ManagerInterface $eventManager,
        SerializerInterface $serializer,
        OrderCollectionFactory $orderCollectionFactory,
        OrderItemCollectionFactory $orderItemCollectionFactory,
        Logger $logger
    ) {
        $this->gateway = $gateway;
        $this->configHelper = $configHelper;
        $this->remoteAddress = $remoteAddress;
        $this->threeDDataFactory = $threeDDataFactory;
        $this->checkoutSession = $checkoutSession;
        $this->quoteData = $quoteData;
        $this->eventManager = $eventManager;
        $this->serializer = $serializer;
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->orderItemCollectionFactory = $orderItemCollectionFactory;
        $this->logger = $logger;
    }

    /**
     * @inheridoc
     */
    public function execute(): ThreeDData
    {
        if (!$this->configHelper->isEnableThreeD()) {
            throw new LocalizedException(
                __('3DS2 is not enabled for this store.')
            );
        }

        $quote = $this->checkoutSession->getQuote();
        $amount = $quote->getGrandTotal() * 100;

        $request = new InitiatePaymentOrderRequest($this->gateway->isLive(), $this->gateway->getCredential(), [
            'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
            'amount' => $amount,
            'ip' => $this->remoteAddress->getRemoteAddress(),
            'orderType' => 'THREED_SECURE',
            'metadata' => $this->gateway->getMetadata(),
        ]);

        $this->logger->debug('3ds init request body: ' . $this->serializer->serialize($request->getRequestData()));

        /** @var PaymentOrderObject $response */
        try {
            $response = $request->execute();
        } catch (Exception $e) {
            $this->logger->debug('3ds init request error: ' . $e->getMessage());
            throw new LocalizedException(
                __('An error occurred while attempting this transaction, please try again or contact support.')
            );
        }

        if ($response instanceof ResponseError) {
            $this->logger->debug('3ds init response error: ' . $this->serializer->serialize($response->toArray()));
            throw new LocalizedException(
                __('An error occurred while attempting this transaction, please try again or contact support.')
            );
        }

        $result = $this->threeDDataFactory->create();
        $result->setOrderToken($response->getOrderToken());
        $result->setProviderClientId($response->getThreedSecureDetails()->getProviderClientId());
        $result->setSessionId($response->getThreedSecureDetails()->getSessionId());
        $result->setSimpleToken($response->getThreedSecureDetails()->getSimpleToken());
        $result->setRiskData($this->serializer->serialize($this->getRiskData($quote)));

        $this->quoteData->setThreeDOrderId($response->getOrderId());

        return $result;
    }

    /**
     * @param Quote $quote
     * @return array
     */
    protected function getRiskData(CartInterface $quote): array
    {
        $skus = [];
        /** @var CartItemInterface $item */
        foreach ($quote->getAllItems() as $item) {
            $skus[] = $item->getSku();
        }
        $orderCollection = $this->orderCollectionFactory->create();
        $orderItemCollection = $this->orderItemCollectionFactory->create();
        $orderCollection->join(
            ['order_item' => $orderItemCollection->getMainTable()],
            '`order_item`.`order_id` = `main_table`.`entity_id`',
            ['order_item.sku']
        )
            ->addFieldToFilter('main_table.customer_email', $quote->getCustomerEmail())
            ->addFieldToFilter('order_item.sku', ['in' => $skus]);

        $riskData = [
            'deliveryTimeframeType' => $quote->isVirtual() ? '01' : '04',
            'reOrderType' => $orderCollection->count() > 0 ? '02' : '01',
        ];
        $riskDataObject = new DataObject($riskData);
        $this->eventManager->dispatch(
            'securepay_3ds_risk_data_get',
            ['risk_data_object' => $riskDataObject, 'quote' => $quote]
        );

        return $riskDataObject->getData();
    }
}
